diff --git a/pgadmin/dlg/dlgColumn.cpp b/pgadmin/dlg/dlgColumn.cpp
index e6178e4..f62bf87 100644
--- a/pgadmin/dlg/dlgColumn.cpp
+++ b/pgadmin/dlg/dlgColumn.cpp
@@ -299,6 +299,15 @@ int dlgColumn::Go(bool modal)
             txtAttstattarget->Disable();
             txtDefault->Disable();
         }
+        else if (table->GetOfTypeOid() > 0)
+        {
+            txtName->Disable();
+            chkNotNull->Enable();
+            txtLength->Disable();
+            cbDatatype->Disable();
+            txtAttstattarget->Enable();
+            txtDefault->Enable();
+        }
 
         size_t i;
         for (i=0 ; i < column->GetVariables().GetCount() ; i++)
diff --git a/pgadmin/dlg/dlgTable.cpp b/pgadmin/dlg/dlgTable.cpp
index b993777..e1207b8 100644
--- a/pgadmin/dlg/dlgTable.cpp
+++ b/pgadmin/dlg/dlgTable.cpp
@@ -41,6 +41,7 @@
 #define btnRemoveTable  CTRL_BUTTON("btnRemoveTable")
 #define cbTables        CTRL_COMBOBOX2("cbTables")
 #define cbTablespace    CTRL_COMBOBOX("cbTablespace")
+#define cbOfType        CTRL_COMBOBOX("cbOfType")
 #define txtFillFactor   CTRL_TEXT("txtFillFactor")
 
 #define btnAddCol       CTRL_BUTTON("btnAddCol")
@@ -102,6 +103,7 @@ BEGIN_EVENT_TABLE(dlgTable, dlgSecurityProperty)
     EVT_CHECKBOX(XRCID("chkHasOids"),               dlgProperty::OnChange)
     EVT_TEXT(XRCID("cbTablespace"),                 dlgProperty::OnChange)
     EVT_COMBOBOX(XRCID("cbTablespace"),             dlgProperty::OnChange)
+    EVT_COMBOBOX(XRCID("cbOfType"),                 dlgTable::OnChangeOfType)
     EVT_CHECKBOX(XRCID("chkHasOids"),               dlgProperty::OnChange)
     EVT_TEXT(XRCID("cbTables"),                     dlgTable::OnChangeTable)
     EVT_BUTTON(XRCID("btnAddTable"),                dlgTable::OnAddTable)
@@ -208,6 +210,16 @@ int dlgTable::Go(bool modal)
     PrepareTablespace(cbTablespace);
     PopulateDatatypeCache();
 
+    // new of type combobox
+    wxString typeQuery = wxT("SELECT t.oid, t.typname ")
+        wxT("FROM pg_type t, pg_namespace n ")
+        wxT("WHERE t.typtype='c' AND t.typnamespace=n.oid ")
+        wxT("AND NOT (n.nspname like 'pg_%' OR n.nspname='information_schema') ")
+        wxT("ORDER BY typname");
+    cbOfType->Insert(wxEmptyString, 0, (void *)0);
+    cbOfType->FillOidKey(connection, typeQuery);
+    cbOfType->SetSelection(0);
+
     hasPK=false;
 
     if (table)
@@ -218,6 +230,9 @@ int dlgTable::Go(bool modal)
         if (table->GetTablespaceOid() != 0)
             cbTablespace->SetKey(table->GetTablespaceOid());
 
+        if (table->GetOfTypeOid() != 0)
+            cbOfType->SetKey(table->GetOfTypeOid());
+
         inheritedTableOids=table->GetInheritedTablesOidList();
 
         wxArrayString qitl=table->GetQuotedInheritedTablesList();
@@ -230,7 +245,6 @@ int dlgTable::Go(bool modal)
 
         btnAddTable->Enable(connection->BackendMinimumVersion(8, 2) && cbTables->GetGuessedSelection() >= 0);
         lbTables->Enable(connection->BackendMinimumVersion(8, 2));
-        cbTables->Enable(connection->BackendMinimumVersion(8, 2));
         chkHasOids->Enable(table->GetHasOids() && connection->BackendMinimumVersion(8, 0));
         cbTablespace->Enable(connection->BackendMinimumVersion(7, 5));
 
@@ -354,6 +368,9 @@ int dlgTable::Go(bool modal)
         cbTablespace->SetSelection(0);
     }
 
+    cbOfType->Enable(connection->BackendMinimumVersion(9, 0) && !table);
+    cbTables->Enable(connection->BackendMinimumVersion(8, 2) && cbOfType->GetCurrentSelection() == 0);
+
     if (connection->BackendMinimumVersion(8,2) || !table)
     {
         wxString systemRestriction;
@@ -1187,26 +1204,34 @@ wxString dlgTable::GetSql()
     }
     else
     {
-        sql = wxT("CREATE TABLE ") + tabname
-            + wxT("\n(");
+        bool typedTable = cbOfType->GetCurrentSelection() > 0 && cbOfType->GetOIDKey() > 0;
+        sql = wxT("CREATE TABLE ") + tabname;
+        if (typedTable)
+            sql += wxT("\nOF ") + qtIdent(cbOfType->GetValue());
+
+        if (!typedTable || (typedTable && lstConstraints->GetItemCount() > 0))
+            sql += wxT("\n(");
 
         int pos;
         bool needComma=false;
-        for (pos=0 ; pos < lstColumns->GetItemCount() ; pos++)
+        if (!typedTable)
         {
-            if (lstColumns->GetText(pos, 2).IsEmpty())
+            for (pos=0 ; pos < lstColumns->GetItemCount() ; pos++)
             {
-                // standard definition, not inherited
-                if (needComma)
-                    sql += wxT(", ");
-                else
-                    needComma=true;
+                if (lstColumns->GetText(pos, 2).IsEmpty())
+                {
+                    // standard definition, not inherited
+                    if (needComma)
+                        sql += wxT(", ");
+                    else
+                        needComma=true;
 
-                wxString name=lstColumns->GetText(pos);
-                wxString definition = lstColumns->GetText(pos, 1);
+                    wxString name=lstColumns->GetText(pos);
+                    wxString definition = lstColumns->GetText(pos, 1);
 
-                sql += wxT("\n   ") + qtIdent(name)
-                    + wxT(" ") + definition;
+                    sql += wxT("\n   ") + qtIdent(name)
+                        + wxT(" ") + definition;
+                }
             }
         }
 
@@ -1225,7 +1250,9 @@ wxString dlgTable::GetSql()
 
             sql += wxT(" ") + GetItemConstraintType(lstConstraints, pos) + wxT(" ") + definition;
         }
-        sql += wxT("\n) ");
+        
+        if (!typedTable || (typedTable && lstConstraints->GetItemCount() > 0))
+            sql += wxT("\n) ");
 
 
         if (lbTables->GetCount() > 0)
@@ -1732,6 +1759,64 @@ void dlgTable::OnSelChangeTable(wxCommandEvent &ev)
 }
 
 
+void dlgTable::OnChangeOfType(wxCommandEvent &ev)
+{
+    if (cbOfType->GetCurrentSelection() > 0)
+    {
+        if (settings->GetConfirmDelete() && lstColumns->GetItemCount() > 0)
+        {
+            if (wxMessageBox(_("A typed table only has the type's columns. All other columns will be dropped. Are you sure you want to do this?"), _("Remove all columns?"), wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION) == wxYES)
+            {
+                lstColumns->DeleteAllItems();
+                lstConstraints->DeleteAllItems();
+                hasPK = false;
+                FillConstraint();
+            }
+            else
+            {
+                cbOfType->SetSelection(0);
+                return;
+            }
+        }
+
+        pgSet *set=connection->ExecuteSet(
+            wxT("SELECT a.attname, format_type(atttypid, NULL) AS atttypname ")
+            wxT("FROM pg_attribute a, pg_class c\n")
+            wxT("WHERE NOT a.attisdropped AND a.attnum>0 ")
+            wxT("AND a.attrelid=c.oid AND c.relname='") + qtIdent(cbOfType->GetValue()) + wxT("'"));
+        if (set)
+        {
+            while (!set->Eof())
+            {
+                lstColumns->AppendItem(tableFactory.GetIconId(),
+                    set->GetVal(wxT("attname")), 
+                    set->GetVal(wxT("atttypname")),
+                    wxT(""));
+                set->MoveNext();
+            }
+            delete set;
+        }        
+    }
+    else
+    {
+        if (settings->GetConfirmDelete() && lstColumns->GetItemCount() > 0)
+        {
+            if (wxMessageBox(_("All type's columns will be dropped. Are you sure you want to do this?"), _("Remove all columns?"), wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION) == wxYES)
+            {
+                lstColumns->DeleteAllItems();
+                lstConstraints->DeleteAllItems();
+                hasPK = false;
+                FillConstraint();
+            }
+        }
+    }
+
+    btnAddCol->Enable(cbOfType->GetCurrentSelection() == 0);
+    cbTables->Enable(connection->BackendMinimumVersion(8, 2) && cbOfType->GetCurrentSelection() == 0);
+    btnRemoveCol->Enable(false);
+    CheckChange();
+}
+
 void dlgTable::OnChangeCol(wxCommandEvent &ev)
 {
     long pos=lstColumns->GetSelection();
@@ -1810,7 +1895,8 @@ void dlgTable::OnSelChangeCol(wxListEvent &ev)
     long pos=lstColumns->GetSelection();
     wxString inheritedFromTable=lstColumns->GetText(pos, 2);
     
-    btnRemoveCol->Enable(inheritedFromTable.IsEmpty());
+    btnAddCol->Enable(!(cbOfType->GetCurrentSelection() > 0 && cbOfType->GetOIDKey() > 0));
+    btnRemoveCol->Enable(inheritedFromTable.IsEmpty() && !(cbOfType->GetCurrentSelection() > 0 && cbOfType->GetOIDKey() > 0));
     btnChangeCol->Enable(table != 0 && !lstColumns->GetText(pos, 6).IsEmpty() && inheritedFromTable.IsEmpty());
 }
 
diff --git a/pgadmin/include/dlg/dlgTable.h b/pgadmin/include/dlg/dlgTable.h
index fb971fa..318d0dc 100644
--- a/pgadmin/include/dlg/dlgTable.h
+++ b/pgadmin/include/dlg/dlgTable.h
@@ -42,6 +42,8 @@ private:
     void OnChangeSize(wxSizeEvent &ev);
 #endif
 
+    void OnChangeOfType(wxCommandEvent &ev);
+
     void OnAddTable(wxCommandEvent &ev);
     void OnRemoveTable(wxCommandEvent &ev);
     void OnSelChangeTable(wxCommandEvent &ev);
diff --git a/pgadmin/include/schema/pgTable.h b/pgadmin/include/schema/pgTable.h
index a453cd0..5626c54 100644
--- a/pgadmin/include/schema/pgTable.h
+++ b/pgadmin/include/schema/pgTable.h
@@ -72,6 +72,10 @@ public:
     void iSetTablespace(const wxString& newVal) { tablespace = newVal; }
     OID GetTablespaceOid() const { return tablespaceOid; };
     void iSetTablespaceOid(const OID newVal) { tablespaceOid = newVal; }
+    wxString GetOfType() const { return ofType; };
+    void iSetOfType(const wxString& newVal) { ofType = newVal; }
+    OID GetOfTypeOid() const { return ofTypeOid; };
+    void iSetOfTypeOid(const OID newVal) { ofTypeOid = newVal; }
     wxULongLong GetRows() const { return rows; }
     long GetInheritedTableCount() { if (inheritedTableCount < 0) UpdateInheritance(); return inheritedTableCount; }
     wxString GetInheritedTables() { GetInheritedTableCount(); return inheritedTables; }
@@ -225,11 +229,13 @@ private:
    
     long inheritedTableCount;
     wxString quotedInheritedTables, inheritedTables, primaryKey, quotedPrimaryKey,
-        primaryKeyName, primaryKeyColNumbers, tablespace, distributionColNumbers;
+        primaryKeyName, primaryKeyColNumbers, tablespace,
+        distributionColNumbers, ofType;
     wxArrayString quotedInheritedTablesList, inheritedTablesOidList;
 
     slSet *replicationSet;
     OID tablespaceOid;
+    OID ofTypeOid;
 };
 
 
diff --git a/pgadmin/schema/pgColumn.cpp b/pgadmin/schema/pgColumn.cpp
index 353880d..9bddcd5 100644
--- a/pgadmin/schema/pgColumn.cpp
+++ b/pgadmin/schema/pgColumn.cpp
@@ -200,9 +200,12 @@ wxString pgColumn::GetPrivileges()
 }
 wxString pgColumn::GetDefinition()
 {
-    wxString sql = GetQuotedTypename();
+    wxString sql = wxEmptyString;
     wxString seqDefault1, seqDefault2;
 
+    if (table->GetOfTypeOid() == 0)
+        sql += GetQuotedTypename();
+
     if (GetDatabase()->BackendMinimumVersion(8, 1))
     {
         seqDefault1 = wxT("nextval('") + schema->GetPrefix() + GetTableName()
diff --git a/pgadmin/schema/pgTable.cpp b/pgadmin/schema/pgTable.cpp
index ed77bc3..99744cb 100644
--- a/pgadmin/schema/pgTable.cpp
+++ b/pgadmin/schema/pgTable.cpp
@@ -206,6 +206,8 @@ wxString pgTable::GetSql(ctlTree *browser)
 {
     wxString colDetails, conDetails;
     wxString prevComment;
+    wxString cols_sql = wxEmptyString;
+    bool showparens = false;
 
     wxString columnPrivileges;
 
@@ -215,7 +217,11 @@ wxString pgTable::GetSql(ctlTree *browser)
         ShowTreeDetail(browser);
         sql = wxT("-- Table: ") + GetQuotedFullIdentifier() + wxT("\n\n")
             + wxT("-- DROP TABLE ") + GetQuotedFullIdentifier() + wxT(";")
-            + wxT("\n\nCREATE TABLE ") + GetQuotedFullIdentifier() + wxT("\n(\n");
+            + wxT("\n\nCREATE TABLE ") + GetQuotedFullIdentifier();
+
+        // of type (9.0 material)
+        if (ofTypeOid > 0)
+            sql += wxT("\nOF ") + qtIdent(ofType);
 
         // Get a count of the constraints.
         int consCount=0;
@@ -256,22 +262,44 @@ wxString pgTable::GetSql(ctlTree *browser)
                     {
                         // Only add a comma if this isn't the last 'real' column, or if there are constraints
                         if (colCount != lastRealCol || consCount)
-                            sql += wxT(",");
+                            cols_sql += wxT(",");
                         if (!prevComment.IsEmpty())
-                            sql += wxT(" -- ") + firstLineOnly(prevComment);
+                            cols_sql += wxT(" -- ") + firstLineOnly(prevComment);
 
-                        sql += wxT("\n");
+                        cols_sql += wxT("\n");
                     }
 
                     if (column->GetInheritedCount() > 0)
                     {
                         if (!column->GetIsLocal())
-                            sql += wxString::Format(wxT("-- %s "), _("Inherited"))
-                                + wxT("from table ") +  column->GetInheritedTableName() + wxT(":");
+                        {
+                            cols_sql += wxString::Format(wxT("-- %s "), _("Inherited"))
+                                     + wxT("from table ") +  column->GetInheritedTableName() + wxT(":");
+                            showparens = true;
+                        }
                     }
 
-                    sql += wxT("  ") + column->GetQuotedIdentifier() + wxT(" ")
-                        + column->GetDefinition();
+                    if (ofTypeOid > 0)
+                    {
+                        if (column->GetDefinition().Length() == 0)
+                        {
+                            cols_sql += wxString::Format(wxT("-- %s "), _("Inherited"))
+                                     + wxT("from type ") +  ofType + wxT(": ")
+                                     + column->GetQuotedIdentifier();
+                        }
+                        else
+                        {
+                            cols_sql += wxT("  ") + column->GetQuotedIdentifier() + wxT(" WITH OPTIONS ")
+                                     + column->GetDefinition();
+                            showparens = true;
+                        }
+                    }
+                    else
+                    {
+                        cols_sql += wxT("  ") + column->GetQuotedIdentifier() + wxT(" ")
+                                 + column->GetDefinition();
+                        showparens = true;
+                    }
 
                     prevComment = column->GetComment();
 
@@ -300,14 +328,14 @@ wxString pgTable::GetSql(ctlTree *browser)
             {
                 data->ShowTreeDetail(browser);
 
-                sql += wxT(",");
+                cols_sql += wxT(",");
 
                 if (!prevComment.IsEmpty())
-                    sql += wxT(" -- ") + firstLineOnly(prevComment);
+                    cols_sql += wxT(" -- ") + firstLineOnly(prevComment);
 
-                sql += wxT("\n  CONSTRAINT ") + data->GetQuotedIdentifier() 
-                    + wxT(" ") + data->GetTypeName().Upper() 
-                    + wxT(" ") ;
+                cols_sql += wxT("\n  CONSTRAINT ") + data->GetQuotedIdentifier() 
+                         + wxT(" ") + data->GetTypeName().Upper() 
+                         + wxT(" ") ;
 
                 prevComment = data->GetComment();
                 if (!data->GetComment().IsEmpty())
@@ -319,21 +347,25 @@ wxString pgTable::GetSql(ctlTree *browser)
                 {
                     case PGM_PRIMARYKEY:
                     case PGM_UNIQUE:
-                        sql += ((pgIndexConstraint*)data)->GetDefinition();
+                        cols_sql += ((pgIndexConstraint*)data)->GetDefinition();
                         break;
                     case PGM_FOREIGNKEY:
-                        sql += ((pgForeignKey*)data)->GetDefinition();
+                        cols_sql += ((pgForeignKey*)data)->GetDefinition();
                         break;
                     case PGM_CHECK:
-                      sql += wxT("(") + ((pgCheck*)data)->GetDefinition() + wxT(")");
+                        cols_sql += wxT("(") + ((pgCheck*)data)->GetDefinition() + wxT(")");
                         break;
                 }
             }
         }
         if (!prevComment.IsEmpty())
-            sql += wxT(" -- ") + firstLineOnly(prevComment);
+            cols_sql += wxT(" -- ") + firstLineOnly(prevComment);
+
+        if (showparens)
+            sql += wxT("\n(\n") + cols_sql + wxT("\n)");
+        else
+            sql += wxT("\n-- (\n") + cols_sql + wxT("\n-- )");
 
-        sql += wxT("\n)");
         if (GetInheritedTableCount())
         {
             sql += wxT("\nINHERITS (") + GetQuotedInheritedTables() + wxT(")");
@@ -828,6 +860,8 @@ void pgTable::ShowTreeDetail(ctlTree *browser, frmMain *form, ctlListView *prope
         if (GetConnection()->BackendMinimumVersion(8, 0))
             properties->AppendItem(_("Tablespace"), tablespace);
         properties->AppendItem(_("ACL"), GetAcl());
+        if (GetConnection()->BackendMinimumVersion(9, 0))
+            properties->AppendItem(_("Of type"), ofType);
         if (GetPrimaryKey().IsNull())
             properties->AppendItem(_("Primary key"), _("<no primary key>"));
         else
@@ -1249,6 +1283,8 @@ pgObject *pgTableFactory::CreateObjects(pgCollection *collection, ctlTree *brows
                      wxT(", rel.reloptions AS reloptions \n")
                      wxT(", (CASE WHEN rel.reltoastrelid = 0 THEN false ELSE true END) AS hastoasttable\n");
         }
+        if (collection->GetConnection()->BackendMinimumVersion(9, 0))
+            query += wxT(", reloftype, typname\n");
 
         query += wxT("  FROM pg_class rel\n")
             wxT("  LEFT OUTER JOIN pg_tablespace ta on ta.oid=rel.reltablespace\n")
@@ -1260,6 +1296,10 @@ pgObject *pgTableFactory::CreateObjects(pgCollection *collection, ctlTree *brows
             //if (collection->GetConnection()->GetIsGreenplum() && collection->GetConnection()->BackendMinimumVersion(8, 2, 9))
             //	query += wxT(" LEFT OUTER JOIN pg_partition ON rel.oid = parrelid\n");
         }
+
+        if (collection->GetConnection()->BackendMinimumVersion(9, 0))
+            query += wxT("LEFT JOIN pg_type ON reloftype=pg_type.oid\n");
+
         query += wxT(" WHERE relkind IN ('r','s','t') AND relnamespace = ") + collection->GetSchema()->GetOidStr() + wxT("\n");
 
         // Greenplum: Eliminate (sub)partitions from the display, only show the parent partitioned table
@@ -1307,6 +1347,16 @@ pgObject *pgTableFactory::CreateObjects(pgCollection *collection, ctlTree *brows
                 else
                     table->iSetTablespace(tables->GetVal(wxT("spcname")));
             }
+            if (collection->GetConnection()->BackendMinimumVersion(9, 0))
+            {
+                table->iSetOfTypeOid(tables->GetOid(wxT("reloftype")));
+                table->iSetOfType(tables->GetVal(wxT("typname")));
+            }
+            else
+            {
+                table->iSetOfTypeOid(0);
+                table->iSetOfType(wxT(""));
+            }
             table->iSetComment(tables->GetVal(wxT("description")));
             table->iSetHasOids(tables->GetBool(wxT("relhasoids")));
             table->iSetEstimatedRows(tables->GetDouble(wxT("reltuples")) * gp_segments);
diff --git a/pgadmin/ui/dlgColumn.xrc b/pgadmin/ui/dlgColumn.xrc
index 7400e77..cd35cf1 100644
--- a/pgadmin/ui/dlgColumn.xrc
+++ b/pgadmin/ui/dlgColumn.xrc
@@ -17,10 +17,10 @@
             <object class="wxPanel" name="pnlProperties">
               <object class="wxFlexGridSizer">
                 <cols>2</cols>
-                <rows>10</rows>
+                <rows>9</rows>
                 <vgap>5</vgap>
                 <hgap>5</hgap>
-                <growablerows>8</growablerows>
+                <growablerows>7</growablerows>
                 <growablecols>1</growablecols>
                 <object class="sizeritem">
                   <object class="wxStaticText" name="stName">
diff --git a/pgadmin/ui/dlgTable.xrc b/pgadmin/ui/dlgTable.xrc
index b94d689..b868a31 100644
--- a/pgadmin/ui/dlgTable.xrc
+++ b/pgadmin/ui/dlgTable.xrc
@@ -17,10 +17,10 @@
             <object class="wxPanel" name="pnlProperties">
               <object class="wxFlexGridSizer">
                 <cols>2</cols>
-                <rows>8</rows>
+                <rows>9</rows>
                 <vgap>5</vgap>
                 <hgap>5</hgap>
-                <growablerows>6</growablerows>
+                <growablerows>7</growablerows>
                 <growablecols>1</growablecols>
                 <object class="sizeritem">
                   <object class="wxStaticText" name="stName">
@@ -77,6 +77,21 @@
                   <border>4</border>
                 </object>
                 <object class="sizeritem">
+                  <object class="wxStaticText" name="stOfType">
+                    <label>Of type</label>
+                  </object>
+                  <flag>wxALIGN_CENTRE_VERTICAL|wxTOP|wxLEFT|wxRIGHT</flag>
+                  <border>4</border>
+                </object>
+                <object class="sizeritem">
+                  <object class="ctlComboBox" name="cbOfType">
+                    <content/>
+                    <style>wxCB_DROPDOWN</style>
+                  </object>
+                  <flag>wxEXPAND|wxALIGN_CENTER_VERTICAL|wxTOP|wxLEFT|wxRIGHT</flag>
+                  <border>4</border>
+                </object>
+                <object class="sizeritem">
                   <object class="wxStaticText" name="stFillFactor">
                     <label>Fill Factor</label>
                   </object>
